@@ -3,26 +3,37 @@ use strict;
 use warnings;
 use Carp qw();
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 use Package::Autoloader::Package;
 my $PACKAGES = {};
 
 my $obtain = sub {
-	my ($name, $visit_point) = @_;
-	unless (exists($PACKAGES->{$name})) {
-		$PACKAGES->{$name} = Package::Autoloader::Package->new
-			($name, $visit_point);
+	my ($pkg_name, $visit_point) = @_;
+	unless (exists($PACKAGES->{$pkg_name})) {
+		$PACKAGES->{$pkg_name} = Package::Autoloader::Package->new
+			($pkg_name, $visit_point);
 	}
-	return($PACKAGES->{$name});
+	return($PACKAGES->{$pkg_name});
 };
 
-
 sub new {
 	my ($class) = (shift);
 	return($obtain->((caller())[0], @_));
 }
 
+sub find_generator($@) {
+	my ($ISA) = (shift);
+
+	foreach my $pkg_name (@$ISA) {
+		next unless (exists($PACKAGES->{$pkg_name}));
+		my $generator = $PACKAGES->{$pkg_name}->find_generator(@_);
+		if (defined($generator)) {
+			return($PACKAGES->{$pkg_name}, $generator);
+		}
+	}
+	return(undef);
+}
 
 sub import {
 	my ($class) = (shift);
@@ -43,11 +54,9 @@ sub import {
 	return;
 }
 
-
 #sub debug_dump {
 #	use Data::Dumper;
 #	print STDERR Dumper($PACKAGES);
 #}
 
-
-1;
+1;
\ No newline at end of file
